@@ -1,4 +1,4 @@
-# pylint: disable=C0111,R0902,R0913
+# pylint: disable=C0111,R0902,R0913,E1137,W3101
 # Smartsheet Python SDK.
 #
 # Copyright 2016 Smartsheet.com, Inc.
@@ -17,16 +17,16 @@
 
 from __future__ import absolute_import
 
-from .models import DownloadedFile
-from .models import Error, ErrorResult
-import requests
 import logging
 import os.path
-import six
+
+import requests
+
 from . import fresh_operation
+from .models import DownloadedFile, Error, ErrorResult
 
 
-class Attachments(object):
+class Attachments:
 
     """Class for handling Attachments operations."""
 
@@ -46,20 +46,20 @@ class Attachments(object):
         Returns:
             Result
         """
-        if not all(val is not None for val in ['sheet_id', 'comment_id',
-                                               '_file']):
+        if not all(val is not None for val in ["sheet_id", "comment_id", "_file"]):
             raise ValueError(
-                ('One or more required values '
-                 'are missing from call to ' + __name__))
+                ("One or more required values are missing from call to " + __name__)
+            )
 
-        _op = fresh_operation('attach_file_to_comment')
-        _op['method'] = 'POST'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/comments/' + str(
-            comment_id) + '/attachments'
-        _op['files'] = {}
-        _op['files']['file'] = _file
+        _op = fresh_operation("attach_file_to_comment")
+        _op["method"] = "POST"
+        _op["path"] = (
+            "/sheets/" + str(sheet_id) + "/comments/" + str(comment_id) + "/attachments"
+        )
+        _op["files"] = {}
+        _op["files"]["file"] = _file
 
-        expected = ['Result', 'Attachment']
+        expected = ["Result", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
@@ -77,19 +77,20 @@ class Attachments(object):
         Returns:
             Result
         """
-        if not all(val is not None for val in ['sheet_id', 'row_id', '_file']):
+        if not all(val is not None for val in ["sheet_id", "row_id", "_file"]):
             raise ValueError(
-                ('One or more required values '
-                 'are missing from call to ' + __name__))
+                ("One or more required values are missing from call to " + __name__)
+            )
 
-        _op = fresh_operation('attach_file_to_row')
-        _op['method'] = 'POST'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/rows/' + str(
-            row_id) + '/attachments'
-        _op['files'] = {}
-        _op['files']['file'] = _file
+        _op = fresh_operation("attach_file_to_row")
+        _op["method"] = "POST"
+        _op["path"] = (
+            "/sheets/" + str(sheet_id) + "/rows/" + str(row_id) + "/attachments"
+        )
+        _op["files"] = {}
+        _op["files"]["file"] = _file
 
-        expected = ['Result', 'Attachment']
+        expected = ["Result", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
@@ -106,13 +107,13 @@ class Attachments(object):
         Returns:
             Result
         """
-        _op = fresh_operation('attach_file_to_sheet')
-        _op['method'] = 'POST'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/attachments'
-        _op['files'] = {}
-        _op['files']['file'] = _file
+        _op = fresh_operation("attach_file_to_sheet")
+        _op["method"] = "POST"
+        _op["path"] = "/sheets/" + str(sheet_id) + "/attachments"
+        _op["files"] = {}
+        _op["files"]["file"] = _file
 
-        expected = ['Result', 'Attachment']
+        expected = ["Result", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
@@ -130,20 +131,24 @@ class Attachments(object):
         Returns:
             Result
         """
-        if not all(val is not None for val in ['sheet_id', 'attachment_id',
-                                               '_file']):
+        if not all(val is not None for val in ["sheet_id", "attachment_id", "_file"]):
             raise ValueError(
-                ('One or more required values '
-                 'are missing from call to ' + __name__))
+                ("One or more required values are missing from call to " + __name__)
+            )
 
-        _op = fresh_operation('attach_new_version')
-        _op['method'] = 'POST'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/attachments/' + str(
-            attachment_id) + '/versions'
-        _op['files'] = {}
-        _op['files']['file'] = _file
+        _op = fresh_operation("attach_new_version")
+        _op["method"] = "POST"
+        _op["path"] = (
+            "/sheets/"
+            + str(sheet_id)
+            + "/attachments/"
+            + str(attachment_id)
+            + "/versions"
+        )
+        _op["files"] = {}
+        _op["files"]["file"] = _file
 
-        expected = ['Result', 'Attachment']
+        expected = ["Result", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
@@ -182,21 +187,21 @@ class Attachments(object):
         Returns:
             Result
         """
-        if not all(val is not None for val in ['sheet_id', 'comment_id',
-                                               'attachment_obj']):
+        if not all(
+            val is not None for val in ["sheet_id", "comment_id", "attachment_obj"]
+        ):
             raise ValueError(
-                ('One or more required values '
-                 'are missing from call to ' + __name__))
+                ("One or more required values are missing from call to " + __name__)
+            )
 
-        _op = fresh_operation('attach_url_to_comment')
-        _op['method'] = 'POST'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/comments/' + str(
-            comment_id) + '/attachments'
-        _op['json'] = attachment_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'attach_url_to_comment'
+        _op = fresh_operation("attach_url_to_comment")
+        _op["method"] = "POST"
+        _op["path"] = (
+            "/sheets/" + str(sheet_id) + "/comments/" + str(comment_id) + "/attachments"
+        )
+        _op["json"] = attachment_obj
 
-        expected = ['Result', 'Attachment']
+        expected = ["Result", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
@@ -235,21 +240,19 @@ class Attachments(object):
         Returns:
             Result
         """
-        if not all(val is not None for val in ['sheet_id', 'row_id',
-                                               'attachment_obj']):
+        if not all(val is not None for val in ["sheet_id", "row_id", "attachment_obj"]):
             raise ValueError(
-                ('One or more required values '
-                 'are missing from call to ' + __name__))
+                ("One or more required values are missing from call to " + __name__)
+            )
 
-        _op = fresh_operation('attach_url_to_row')
-        _op['method'] = 'POST'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/rows/' + str(
-            row_id) + '/attachments'
-        _op['json'] = attachment_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'attach_url_to_row'
+        _op = fresh_operation("attach_url_to_row")
+        _op["method"] = "POST"
+        _op["path"] = (
+            "/sheets/" + str(sheet_id) + "/rows/" + str(row_id) + "/attachments"
+        )
+        _op["json"] = attachment_obj
 
-        expected = ['Result', 'Attachment']
+        expected = ["Result", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
@@ -287,14 +290,12 @@ class Attachments(object):
         Returns:
             Result
         """
-        _op = fresh_operation('attach_url_to_sheet')
-        _op['method'] = 'POST'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/attachments'
-        _op['json'] = attachment_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'attach_url_to_sheet'
+        _op = fresh_operation("attach_url_to_sheet")
+        _op["method"] = "POST"
+        _op["path"] = "/sheets/" + str(sheet_id) + "/attachments"
+        _op["json"] = attachment_obj
 
-        expected = ['Result', 'Attachment']
+        expected = ["Result", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
@@ -311,12 +312,11 @@ class Attachments(object):
         Returns:
             Result
         """
-        _op = fresh_operation('delete_attachment')
-        _op['method'] = 'DELETE'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/attachments/' + str(
-            attachment_id)
+        _op = fresh_operation("delete_attachment")
+        _op["method"] = "DELETE"
+        _op["path"] = "/sheets/" + str(sheet_id) + "/attachments/" + str(attachment_id)
 
-        expected = 'Result'
+        expected = ["Result", None]
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
@@ -335,12 +335,17 @@ class Attachments(object):
         Returns:
             Result
         """
-        _op = fresh_operation('delete_attachment_versions')
-        _op['method'] = 'DELETE'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/attachments/' + str(
-            attachment_id) + '/versions'
+        _op = fresh_operation("delete_attachment_versions")
+        _op["method"] = "DELETE"
+        _op["path"] = (
+            "/sheets/"
+            + str(sheet_id)
+            + "/attachments/"
+            + str(attachment_id)
+            + "/versions"
+        )
 
-        expected = 'Result'
+        expected = ["Result", None]
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
@@ -356,19 +361,19 @@ class Attachments(object):
         Returns:
             Attachment
         """
-        _op = fresh_operation('get_attachment')
-        _op['method'] = 'GET'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/attachments/' + str(
-            attachment_id)
+        _op = fresh_operation("get_attachment")
+        _op["method"] = "GET"
+        _op["path"] = "/sheets/" + str(sheet_id) + "/attachments/" + str(attachment_id)
 
-        expected = 'Attachment'
+        expected = "Attachment"
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
         return response
 
-    def list_all_attachments(self, sheet_id, page_size=100, page=1,
-                             include_all=False):
+    def list_all_attachments(
+        self, sheet_id, page_size=None, page=None, include_all=None
+    ):
         """Get a list of Attachments for a Sheet.
 
         Get a list of all Attachments for the specified Sheet,
@@ -377,31 +382,31 @@ class Attachments(object):
         Args:
             sheet_id (int): Sheet ID
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
 
         Returns:
             IndexResult
         """
-        _op = fresh_operation('list_all_attachments')
-        _op['method'] = 'GET'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/attachments'
-        _op['query_params']['pageSize'] = page_size
-        _op['query_params']['page'] = page
-        _op['query_params']['includeAll'] = include_all
+        _op = fresh_operation("list_all_attachments")
+        _op["method"] = "GET"
+        _op["path"] = "/sheets/" + str(sheet_id) + "/attachments"
+        _op["query_params"]["pageSize"] = page_size
+        _op["query_params"]["page"] = page
+        _op["query_params"]["includeAll"] = include_all
 
-        expected = ['IndexResult', 'Attachment']
+        expected = ["IndexResult", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
         return response
 
-    def list_attachment_versions(self, sheet_id, attachment_id,
-                                 page_size=100, page=1, include_all=False):
+    def list_attachment_versions(
+        self, sheet_id, attachment_id, page_size=None, page=None, include_all=None
+    ):
         """Get a list of versions for an Attachment.
 
         Get a list of all versions of the given Attachment ID, in
@@ -411,32 +416,37 @@ class Attachments(object):
             sheet_id (int): Sheet ID
             attachment_id (int): Attachment ID
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
 
         Returns:
             IndexResult
         """
-        _op = fresh_operation('list_attachment_versions')
-        _op['method'] = 'GET'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/attachments/' + str(
-            attachment_id) + '/versions'
-        _op['query_params']['pageSize'] = page_size
-        _op['query_params']['page'] = page
-        _op['query_params']['includeAll'] = include_all
+        _op = fresh_operation("list_attachment_versions")
+        _op["method"] = "GET"
+        _op["path"] = (
+            "/sheets/"
+            + str(sheet_id)
+            + "/attachments/"
+            + str(attachment_id)
+            + "/versions"
+        )
+        _op["query_params"]["pageSize"] = page_size
+        _op["query_params"]["page"] = page
+        _op["query_params"]["includeAll"] = include_all
 
-        expected = ['IndexResult', 'Attachment']
+        expected = ["IndexResult", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
         return response
 
-    def list_discussion_attachments(self, sheet_id, discussion_id,
-                                    page_size=100, page=1, include_all=False):
+    def list_discussion_attachments(
+        self, sheet_id, discussion_id, page_size=None, page=None, include_all=None
+    ):
         """Get a list of Attachments for the Sheet Discussion.
 
         Get a list of all Attachments for the specified Sheet
@@ -446,64 +456,70 @@ class Attachments(object):
             sheet_id (int): Sheet ID
             discussion_id (int): Discussion ID
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
 
         Returns:
             IndexResult
         """
-        _op = fresh_operation('list_discussion_attachments')
-        _op['method'] = 'GET'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/discussions/' + str(
-            discussion_id) + '/attachments'
-        _op['query_params']['pageSize'] = page_size
-        _op['query_params']['page'] = page
-        _op['query_params']['includeAll'] = include_all
+        _op = fresh_operation("list_discussion_attachments")
+        _op["method"] = "GET"
+        _op["path"] = (
+            "/sheets/"
+            + str(sheet_id)
+            + "/discussions/"
+            + str(discussion_id)
+            + "/attachments"
+        )
+        _op["query_params"]["pageSize"] = page_size
+        _op["query_params"]["page"] = page
+        _op["query_params"]["includeAll"] = include_all
 
-        expected = ['IndexResult', 'Attachment']
+        expected = ["IndexResult", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
         return response
 
-    def list_row_attachments(self, sheet_id, row_id, page_size=100, page=1,
-                             include_all=False):
+    def list_row_attachments(
+        self, sheet_id, row_id, page_size=None, page=None, include_all=None
+    ):
         """Get a list of all Attachments for the specified Sheet Row.
 
         Args:
             sheet_id (int): Sheet ID
             row_id (int): Row ID
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
 
         Returns:
             IndexResult
         """
-        _op = fresh_operation('list_row_attachments')
-        _op['method'] = 'GET'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/rows/' + str(
-            row_id) + '/attachments'
-        _op['query_params']['pageSize'] = page_size
-        _op['query_params']['page'] = page
-        _op['query_params']['includeAll'] = include_all
+        _op = fresh_operation("list_row_attachments")
+        _op["method"] = "GET"
+        _op["path"] = (
+            "/sheets/" + str(sheet_id) + "/rows/" + str(row_id) + "/attachments"
+        )
+        _op["query_params"]["pageSize"] = page_size
+        _op["query_params"]["page"] = page
+        _op["query_params"]["includeAll"] = include_all
 
-        expected = ['IndexResult', 'Attachment']
+        expected = ["IndexResult", "Attachment"]
 
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
         return response
 
-    def download_attachment(self, attachment_obj, download_path,
-                            alternate_file_name=None):
+    def download_attachment(
+        self, attachment_obj, download_path, alternate_file_name=None
+    ):
         """Download the specified attachment as a file.
 
         This method wraps the Requests module and performs
@@ -520,21 +536,20 @@ class Attachments(object):
             DownloadedFile
         """
         if not os.path.isdir(download_path):
-            raise ValueError('download_path must be a directory.')
+            raise ValueError("download_path must be a directory.")
 
-        resp = requests.get(
-            attachment_obj.url,
-            stream=True
-        )
+        resp = requests.get(attachment_obj.url, stream=True)
 
         if 200 <= resp.status_code <= 299:
-            response = DownloadedFile({
-                'result_code': 0,
-                'message': 'SUCCESS',
-                'resp': resp,
-                'filename': attachment_obj.name,
-                'download_directory': download_path
-            })
+            response = DownloadedFile(
+                {
+                    "result_code": 0,
+                    "message": "SUCCESS",
+                    "resp": resp,
+                    "filename": attachment_obj.name,
+                    "download_directory": download_path,
+                }
+            )
 
             if alternate_file_name is not None:
                 response.filename = alternate_file_name
@@ -542,9 +557,9 @@ class Attachments(object):
             response.save_to_file()
             return response
         else:
-            return Error({
-                'result': ErrorResult({
-                    'status_code': resp.status_code
-                }),
-                'request_response': resp
-            })
+            return Error(
+                {
+                    "result": ErrorResult({"status_code": resp.status_code}),
+                    "request_response": resp,
+                }
+            )
