@@ -1,4 +1,4 @@
-# pylint: disable=C0111,R0902,R0913
+# pylint: disable=C0111,R0902,R0913,W4904,W0706
 # Smartsheet Python SDK.
 #
 # Copyright 2016 Smartsheet.com, Inc.
@@ -15,19 +15,24 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import collections
+try:
+    # For Python versions 2.7 and 3.3 to 3.9, import from collections
+    from collections import MutableSequence
+except ImportError:
+    # For Python version >= 3.10 import from collections.abc
+    from collections.abc import MutableSequence
+
 import importlib
 import json
 import logging
-import six
-
 from datetime import datetime
-from dateutil.parser import parse
 from enum import Enum
 
+import six
+from dateutil.parser import parse
 
-class TypedList(collections.MutableSequence):
 
+class TypedList(MutableSequence):
     def __init__(self, item_type):
         self.item_type = item_type
         self.__store = []
@@ -35,8 +40,10 @@ class TypedList(collections.MutableSequence):
         if isinstance(self.item_type, six.string_types):
             self.item_type = getattr(
                 importlib.import_module(
-                    __package__ + '.models.' + self.item_type.lower()
-                ), self.item_type)
+                    __package__ + ".models." + self.item_type.lower()
+                ),
+                self.item_type,
+            )
 
     def __len__(self):
         return len(self.__store)
@@ -45,7 +52,7 @@ class TypedList(collections.MutableSequence):
         return self.__store[idx]
 
     def __setitem__(self, idx, value):
-        self._log.debug('__setitem__, %s, %s', idx, value)
+        self._log.debug("__setitem__, %s, %s", idx, value)
         self.__store[idx] = self.convert(value)
 
     def __delitem__(self, idx):
@@ -60,19 +67,21 @@ class TypedList(collections.MutableSequence):
             if isinstance(item, self.item_type):
                 return item
             # allow explicit null to be passed through to the list
-            elif hasattr(item, 'is_explicit_null'):
+            elif hasattr(item, "is_explicit_null"):
                 return item
         except TypeError:
             raise
 
         try:
             retval = self.item_type(item)
-            self._log.debug('item converted to %s: %s -> %s',
-                            self.item_type, item, retval)
+            self._log.debug(
+                "item converted to %s: %s -> %s", self.item_type, item, retval
+            )
             return retval
-        except (ValueError, TypeError):
+        except (ValueError, TypeError) as exc:
             raise ValueError(
-                "Can't convert %s to %s in TypedList", item, self.item_type)
+                f"Can't convert {item} to {self.item_type} in TypedList"
+            ) from exc
 
     def purge(self):
         """Zero out the underlying list object."""
@@ -84,31 +93,35 @@ class TypedList(collections.MutableSequence):
     def load(self, value):
         if isinstance(value, list):
             self.purge()
-            self.extend([
-                (item if isinstance(item, self.item_type) else self.item_type(item)) for item in value
-            ])
+            self.extend(
+                [
+                    (item if isinstance(item, self.item_type) else self.item_type(item))
+                    for item in value
+                ]
+            )
         elif isinstance(value, TypedList):
             self.purge()
             self.extend(value.to_list())
         elif isinstance(value, self.item_type):
             self.purge()
             self.append(value)
-        elif hasattr(value, 'is_explicit_null'):
+        elif hasattr(value, "is_explicit_null"):
             self.purge()
             self.append(value)
         else:
-            raise ValueError("Can't load to TypedList(%s) from '%s'", self.item_type, value)
+            raise ValueError(
+                f"Can't load to TypedList({self.item_type}) from '{value}'"
+            )
 
     def __repr__(self):
         tmp = json.dumps(self.__store)
-        return "TypedList(item_type=%s, contents=%s)" % (self.item_type, tmp)
+        return f"TypedList(item_type={self.item_type}, contents={tmp})"
 
     def __str__(self):
         return json.dumps(self.__store)
 
 
-class TypedObject(object):
-
+class TypedObject:
     def __init__(self, object_type):
         self.object_type = object_type
         self._value = None
@@ -116,8 +129,10 @@ class TypedObject(object):
         if isinstance(self.object_type, six.string_types):
             self.object_type = getattr(
                 importlib.import_module(
-                    __package__ + '.models.' + self.object_type.lower()
-                ), self.object_type)
+                    __package__ + ".models." + self.object_type.lower()
+                ),
+                self.object_type,
+            )
 
     @property
     def value(self):
@@ -129,17 +144,18 @@ class TypedObject(object):
             self._value = value
         elif isinstance(value, dict):
             self._value = self.object_type(value)
-        elif hasattr(value, 'is_explicit_null'):
+        elif hasattr(value, "is_explicit_null"):
             self._value = value
         else:
-            raise ValueError("`{0}` invalid type for {1} value".format(value, self.object_type))
+            raise ValueError(
+                f"`{value}` invalid type for {self.object_type} value"
+            )
 
     def __str(self):
         return json.dumps(self._value)
 
 
-class Number(object):
-
+class Number:
     def __init__(self, initial_value=None):
         self._value = None
         if initial_value:
@@ -156,14 +172,13 @@ class Number(object):
         elif isinstance(value, (six.integer_types, float)):
             self._value = value
         else:
-            raise ValueError("`{0}` invalid type for Number value".format(value))
+            raise ValueError(f"`{value}` invalid type for Number value")
 
     def __str__(self):
         return str(self.value)
 
 
-class String(object):
-
+class String:
     def __init__(self, initial_value=None, accept=None):
         self._value = None
         if initial_value:
@@ -183,11 +198,11 @@ class String(object):
         elif isinstance(value, six.string_types):
             if self.accept and value not in self._accept:
                 raise ValueError(
-                    "`{0}` is not in accept list, must be one of {1}".format(
-                        value, self.accept))
+                    f"`{value}` is not in accept list, must be one of {self.accept}"
+                )
             self._value = value
         else:
-            raise ValueError("`{0}` invalid type for String value".format(value))
+            raise ValueError(f"`{value}` invalid type for String value")
 
     @property
     def accept(self):
@@ -200,14 +215,13 @@ class String(object):
         elif isinstance(value, six.string_types):
             self._accept = [value]
         else:
-            raise ValueError("`{0}` invalid type for accept".format(value))
+            raise ValueError(f"`{value}` invalid type for accept")
 
     def __str__(self):
         return self._value
 
 
-class Boolean(object):
-
+class Boolean:
     def __init__(self, initial_value=None):
         self._value = None
         if initial_value:
@@ -224,14 +238,13 @@ class Boolean(object):
         elif isinstance(value, bool):
             self._value = value
         else:
-            raise ValueError("`{0}` invalid type for Boolean value".format(value))
+            raise ValueError(f"`{value}` invalid type for Boolean value")
 
     def __str__(self):
         return str(self._value)
 
 
-class Timestamp(object):
-
+class Timestamp:
     def __init__(self, initial_value=None):
         self._value = None
         if initial_value:
@@ -251,14 +264,13 @@ class Timestamp(object):
             value = parse(value)
             self._value = value
         else:
-            raise ValueError("`{0}` invalid type for Timestamp value".format(value))
+            raise ValueError(f"`{value}` invalid type for Timestamp value")
 
     def __str__(self):
         return str(self._value)
 
 
-class EnumeratedValue(object):
-
+class EnumeratedValue:
     def __init__(self, enum, value=None):
         self.__enum = enum
         self._value = None
@@ -276,16 +288,16 @@ class EnumeratedValue(object):
             except KeyError:
                 self._value = None
         elif isinstance(value, Enum):
-            self._value = value;
+            self._value = value
         else:
             self._value = None
 
     def __eq__(self, other):
-        if isinstance(other, Enum):
+        if isinstance(other, Enum) or other is None:
             return self._value == other
         elif isinstance(other, six.string_types):
             return self._value == self.__enum[other]
-        NotImplemented
+        return NotImplemented
 
     def __str__(self):
         if self._value is not None:
@@ -295,9 +307,8 @@ class EnumeratedValue(object):
 
 
 class EnumeratedList(TypedList):
-
     def __init__(self, enum):
-        super(EnumeratedList, self).__init__(EnumeratedValue)
+        super().__init__(EnumeratedValue)
         self.__enum = enum
 
     def load(self, value):
@@ -306,9 +317,7 @@ class EnumeratedList(TypedList):
 
         if isinstance(value, list):
             self.purge()
-            self.extend([
-                (EnumeratedValue(self.__enum, item)) for item in value
-            ])
+            self.extend([(EnumeratedValue(self.__enum, item)) for item in value])
         else:
             self.purge()
             self.append(EnumeratedValue(self.__enum, value))
